# 组件概念初探，利用组件改写Todo List
## v-bind
首先来说一下之前经常提到的`{{}}`被称为**插值表达值**, 它的作用是在HTML标签中显示绑定的数据，如：
```HTML
<button>{{content}}</button>
```
但是当需要将数据绑定到HTML标签中的某一属性时，则需要使用`v-bind`这个vue指令。如：
```HTML
<button v-bind:title="btnTitle" >{{content}}</button>
```
其中，v-bind是用于将数据和数据绑定的vue指令； title是需要和数据绑定属性； btnTitle是绑定到title属性的变量。

## 组件
**组件**：页面上的一部分，像淘宝这种展示非常复杂的网站，一个页面往往是是由各个组件组成的，比如导航组件，分页组件，瀑布流展示组件等等。

因此当一个前端项目较为复杂时，经常能听到前端的同学说拆组件，这里说的组件，就是指的页面的一部分，<u>这一部分有独立的数据，逻辑处理和展示模板</u>。

如果看按照组件是页面上的一部分这个逻辑来看，其实我们使用的每一个HTML标签都是一个组件，如`<button>` ,`<input>`...


## 使用组件改写Todo List
这里尝试将代办列表中插入的每一个代办项封装为一个单独的组件。
1. 注册一个组件，并定义这个组件的数据函数 ，函数字域，模板域等
```HTML
 // 实例化Vue实例
 const app = Vue.createApp({
    ...
 });

 // 向app注册名字为todo-item的组件
 app.component('todo-item',{
     // 这里定义组建的数据域、函数域、模板域
     // 组件内部数据域
     data(){
         return{
         }
     },
     // 外部向组件传递的值
     props:['content',"content_index"],
     // 模板域
     template:`<li>{{content_index}} - {{content}}</li>`
 });
```
通过上面代码，需要注意有以下需要注意的事项
- 组件的定义需要基于一个VUE实例

- 注册组件时需要指定组件的数据域、函数域和模板域，这一点和Vue实例定义的的步骤一致，因此，可以形而上地将组件看成是一个Vue的实例。

- 组件中的`props`用于接收，使用该组件的Vue实例通过`v-bind`执行向组件中传递的值。 这里可以将组件看做一个函数，在传值时有 v-bind:形参=”实参“。

> Question 1： 当在组件中改变传递的数据的值，父组件中的值是有也会发生变化？

> Question 2： 组件中的数据域中能否被外部组件的调用方获取到


2. 将组件注册到Vue实例上
```HTML
 // 第一步 实例化Vue实例
 const app = Vue.createApp({
    ...
 });

 // 第二步 向app注册名字为todo-item的组件
 app.component('todo-item',{
   ...
 });

 // 第三步 最后才能将app进行挂载
 app.mount('#root')

```
示例代码中的步骤不能错，**一定是组件注册到Vue实例之后，才将Vue实例挂载到Html元素上**

3. 在Vue实例中使用自定义组件
```HTML
<body>
    <!--2.设置DIV的ID为root -->
    <div id="root"></div>
</body>

<script>
    // 3. 创建VUE的实例
    const app = Vue.createApp({
        // 3.1定义需要展示的数据
        data(){
            return{
                // 定义需要列表中展示的内容数组
                list:[], // 默认是一个空数字，展示空列表
                newItemContent:''
            }
        },
        // 3.2 函数域，定义新增代办事项的处理逻辑
        methods:{
            addTodoItem(){
                // 增加一个代办事项
                this.list.push(this.newItemContent);
                // 将输入框清空
                this.newItemContent='';
            }
        },
        // 3.3视图模板，定义显示内容和事件绑定
        template:`
        <input v-model="newItemContent"/>
        <button v-on:click="addTodoItem">增加</button>
        <ul>
            <todo-item
                v-for="(item,index) of list"
                v-bind:content="item"
                v-bind:content_index="index"
            />
        </ul>
        `
    });
```
改写后的能实现和之前一样的功能，效果和万恒代码如下：
![todo_list](./images/todo_list.gif)
```HTMl
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>组件改写ToDo</title>
    <!--1.通过CDN加速引入VUE-->
    <script src="https://unpkg.com/vue@next"></script>
</head>
<body>
    <!--2.设置DIV的ID为root -->
    <div id="root"></div>
</body>

<script>
    // 3. 创建VUE的实例
    const app = Vue.createApp({
        // 3.1定义需要展示的数据
        data(){
            return{
                // 定义需要列表中展示的内容数组
                list:[], // 默认是一个空数字，展示空列表
                newItemContent:''
            }
        },
        // 3.2 函数域，定义新增代办事项的处理逻辑
        methods:{
            addTodoItem(){
                // 增加一个代办事项
                this.list.push(this.newItemContent);
                // 将输入框清空
                this.newItemContent='';
            }
        },
        // 3.3视图模板，定义显示内容和事件绑定
        template:`
        <input v-model="newItemContent"/>
        <button v-on:click="addTodoItem">增加</button>
        <ul>
            <todo-item
                v-for="(item,index) of list"
                v-bind:content="item"
                v-bind:content_index="index"
            />
        </ul>
        `
    });

    // 向app注册名字为todo-item的组件
    app.component('todo-item',{
        // 这里定义组建的数据域、函数域、模板域
        // 组件内部数据域
        data(){
            return{
            }
        },
        // 外部向组件传递的值
        props:['content',"content_index"],
        // 模板域
        template:`<li>{{content_index}} - {{content}}</li>`
    });

    // 最后才能将app进行挂载
    app.mount('#root')

</script>
</html>

```
